﻿/*
 * Copyright 2011 Arne Vajhøj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

using System;
using System.Collections.Generic;

using Vajhoej.StaticWeaver;

namespace Demo
{
    [Aspect]
    public class TestAspect
    {
        [Pointcut(Signature="* *::*(*)")]
        public static void TestPointcut() { }
        [BeforeCallAdvice(Pointcut="TestPointcut")]
        public static void EntryLog(MethodJoinpoint mjp)
        {
            Console.WriteLine("Enter " + mjp.TypeName + "." + mjp.MethodName);
            int ix = 1;
            foreach(object o in mjp.Arguments)
            {
                Console.WriteLine("  arg " + ix + " = " + o);
                ix++;
            }
        }
        [AfterCallAdvice(Pointcut="TestPointcut")]
        public static void ExitLog(MethodJoinpoint mjp)
        {
            Console.WriteLine("Exit " + mjp.TypeName + "." + mjp.MethodName);
            if(mjp.Result != null) Console.WriteLine("  retval = " + mjp.Result);
        }
        [AtExceptionAdvice(Pointcut="TestPointcut")]
        public static void ExceptionLog(MethodJoinpoint mjp)
        {
            Console.WriteLine("Exception in " + mjp.TypeName + "." + mjp.MethodName);
            Console.WriteLine("  exception = " + mjp.CaugthException);
        }
    }
    public class G<T>
    {
        public T Nop(T o)
        {
            return o;
        }
    }
    public class Test
    {
        public static string SP { get; set; }
        public string IP { get; set; }
        public static void SE()
        {
            throw new Exception("Oops");
        }
        public void IE()
        {
            throw new Exception("Oops");
        }
        public static List<string> SG(List<string> a)
        {
            return null;
        }
        public List<string> IG(List<string> a)
        {
            return null;
        }
        public int NVI2(int iv, string sv)
        {
            return 2;
        }
        public int NVI1(int iv)
        {
            return NVI2(iv, "ABC") - 1;
        }
        public int NVI0()
        {
            return NVI1(123) - 1;
        }
        public void VI2(int iv, string sv)
        {
        }
        public void VI1(int iv)
        {
            VI2(iv, "ABC");
        }
        public void VI0()
        {
            VI1(123);
        }
        public static int NVS2(int iv, string sv)
        {
            return 2;
        }
        public static int NVS1(int iv)
        {
            return NVS2(iv, "ABC") - 1;
        }
        public static int NVS0()
        {
            return NVS1(123) - 1;
        }
        public static void VS2(int iv, string sv)
        {
        }
        public static void VS1(int iv)
        {
            VS2(iv, "ABC");
        }
        public static void VS0()
        {
            VS1(123);
        }
        public static void Main(string[] args) 
        {
            VS0();
            NVS0();
            Test o = new Test();
            o.VI0();
            o.NVI0();
            o.IP = "X";
            string s1 = o.IP;
            SP = "Y";
            string s2 = SP;
            SG(null);
            o.IG(null);
            SE();
            o.IE();
            //G<string> go = new G<string>();
            //go.Nop("ABC");
        }
    }
}